#define MOZ_SOURCE_STAMP ea6157b86ae2b200ca3433d462516bf75559e056
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-release
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-release/rev/ea6157b86ae2b200ca3433d462516bf75559e056
